==> 2.0.2.tar.gz: lib/data/repositories/auth_repository.dart <==
import 'dart:convert';
import 'package:dfapi_auth/models/dfapi_user_info.dart';
import 'package:dfapi_auth/models/login_model.dart';
import 'package:dfapi_auth/models/refresh_token_response.dart';
import 'package:flutter_secure_storage/flutter_secure_storage.dart';
import 'package:get_it/get_it.dart';
import 'package:http/http.dart' as http;
import 'package:custom_response/custom_response.dart';

import '../../models/auth_configuration.dart';
import '../../models/authentication_response.dart';
import '../repository_contracts/auth_repository_contract.dart';

class AuthRepository implements AuthRepositoryContract {
  final AuthConfig config;

  final String _tokenKey = "DfApiAuthTokenKey";
  final String _refreshTokenKey = "DfApiAuthRefreshTokenKey";
  final String _tokenExpireDateKey = "DfApiAuthTokenExpireDateKey";
  final String _userInfoKey = "DfApiAuthUserInfoKey";

  final String _usernameKey = "DfApiAuthUsernameKey";
  final String _passwordKey = "DfApiAuthPasswordKey";

  FlutterSecureStorage _secureStorage;

  AuthRepository(this.config)
      : _secureStorage = GetIt.I<FlutterSecureStorage>();

  @override
  Future<Response<AuthenticationResponse>> authenticate(
      LoginModel model) async {
    try {
      var data = {
        "Username": model.username,
        "Password": model.password,
      };
      final http.Response loginResponse = await http.post(
        Uri.parse(config.loginUrl),
        headers: {"content-type": "application/json"},
        body: jsonEncode(data),
      );

      if (loginResponse.statusCode != 200)
        return Response.error(
          "Unexpected Error Has Occured!\n" +
              "STATUS CODE: ${loginResponse.statusCode}\n" +
              "${loginResponse.reasonPhrase}",
        );

      var response =
          AuthenticationResponse.fromJson(jsonDecode(loginResponse.body));

      if (!response.isSuccess) return Response.error(response.errorMessage);

      await _secureStorage.write(key: _tokenKey, value: response.token);

      await _secureStorage.write(
        key: _refreshTokenKey,
        value: response.refreshToken,
      );

      await _secureStorage.write(
        key: _tokenExpireDateKey,
        value: response.tokenExpireDate.millisecondsSinceEpoch.toString(),
      );

      await _secureStorage.write(
        key: _userInfoKey,
        value: jsonEncode(response.userInfo.asJson),
      );
      await _secureStorage.write(key: _usernameKey, value: model.username);
      await _secureStorage.write(key: _passwordKey, value: model.password);

      return Response.success(response);
    } catch (e) {
      return Response.error(
        "Unexpected error has occured while authetication ... \n\n" +
            e.toString(),
      );
    }
  }

  @override
  Future<Response> logOut() async {
    try {
      var token = await _secureStorage.read(key: _tokenKey);
      // var token = _pref.getString(_tokenKey);

      await clearData();

      if (!config.isRootPath(config.logutUrl)) {
        await http.get(
          Uri.parse(config.logutUrl),
          headers: {"Authorization": "Bearer $token"},
        );
      }
      return Response();
    } catch (e) {
      return Response.error(
        "Unexpected error has occured while logout ... \n\n" + e.toString(),
      );
    }
  }

  @override
  Future clearData() async {
    await _secureStorage.deleteAll();
  }

  @override
  Future<Response<AuthenticationResponse>> getAuthData() async {
    if (!await _secureStorage.containsKey(key: _tokenExpireDateKey))
      return Response.error("Token expire date not found!");

    var expireData = await _secureStorage.read(key: _tokenExpireDateKey);
    var tokenExpireDate = DateTime.fromMillisecondsSinceEpoch(
      int.parse(expireData),
    );

    if (DateTime.now().isAfter(tokenExpireDate))
      return Response.error("Token Expired");

    if (!(await _secureStorage.containsKey(key: _tokenKey)))
      return Response.error("Unauthorized User!");

    var token = await _secureStorage.read(key: _tokenKey);

    if (!(await _secureStorage.containsKey(key: _refreshTokenKey)))
      return Response.error("Refresh token not found! ");

    var refreshToken = await _secureStorage.read(key: _refreshTokenKey);

    if (!await _secureStorage.containsKey(key: _userInfoKey))
      return Response.error("User data not found!");

    var userInfoJson = await _secureStorage.read(key: _userInfoKey);
    var userInfo = DfApiUserInfo.fromJson(
      jsonDecode(userInfoJson),
    );

    return Response.success(
      AuthenticationResponse(
        token,
        userInfo,
        refreshToken,
        tokenExpireDate,
      ),
    );
  }

  @override
  Future<Response<AuthenticationResponse>> refreshToken() async {
    try {
      var authDataResponse = await getAuthData();
      if (!authDataResponse.isSuccess)
        return Response.error(authDataResponse.errorMessage);

      var data = {"RefreshToken": authDataResponse.value.refreshToken};

      final http.Response refreshTokenResponse = await http.post(
        Uri.parse(config.refreshTokenUrl),
        headers: {
          "content-type": "application/json",
          "Authorization": "Bearer ${authDataResponse.value.token}"
        },
        body: jsonEncode(data),
      );

      if (refreshTokenResponse.statusCode != 200)
        return Response.error(
          "Unexpected Error Has Occured!\n" +
              "STATUS CODE: ${refreshTokenResponse.statusCode}\n" +
              "${refreshTokenResponse.reasonPhrase}",
        );

      var response = RefreshTokenResponse.fromJson(
        jsonDecode(refreshTokenResponse.body),
      );

      await _secureStorage.write(key: _tokenKey, value: response.token);
      await _secureStorage.write(
        key: _refreshTokenKey,
        value: response.refreshToken,
      );
      await _secureStorage.write(
        key: _tokenExpireDateKey,
        value: response.tokenExpireDate.millisecondsSinceEpoch.toString(),
      );

      authDataResponse.value.token = response.token;
      authDataResponse.value.refreshToken = response.refreshToken;
      authDataResponse.value.tokenExpireDate = response.tokenExpireDate;

      return authDataResponse;
    } catch (e) {
      return Response.error(e.toString());
    }
  }

  @override
  Future<Response<LoginModel>> getLoginData() async {
    var isValid = await _secureStorage.containsKey(key: _usernameKey) &&
        await _secureStorage.containsKey(key: _passwordKey);

    if (!isValid) return Response.error("User data not found!");

    var username = await _secureStorage.read(key: _usernameKey);
    var password = await _secureStorage.read(key: _passwordKey);

    return Response.success(LoginModel(username, password));
  }
}

==> 2.0.7.tar.gz: lib/data/repositories/auth_repository.dart <==
import 'dart:convert';
import 'package:dfapi_auth/models/dfapi_user_info.dart';
import 'package:dfapi_auth/models/login_model.dart';
import 'package:dfapi_auth/models/refresh_token_response.dart';
import 'package:flutter_secure_storage/flutter_secure_storage.dart';
import 'package:get_it/get_it.dart';
import 'package:http/http.dart' as http;
import 'package:custom_response/custom_response.dart';

import '../../models/auth_configuration.dart';
import '../../models/authentication_response.dart';
import '../repository_contracts/auth_repository_contract.dart';

class AuthRepository implements AuthRepositoryContract {
  final AuthConfig config;

  final String _tokenKey = "DfApiAuthTokenKey";
  final String _refreshTokenKey = "DfApiAuthRefreshTokenKey";
  final String _tokenExpireDateKey = "DfApiAuthTokenExpireDateKey";
  final String _userInfoKey = "DfApiAuthUserInfoKey";

  final String _usernameKey = "DfApiAuthUsernameKey";
  final String _passwordKey = "DfApiAuthPasswordKey";

  FlutterSecureStorage _secureStorage;

  AuthRepository(this.config)
      : _secureStorage = GetIt.I<FlutterSecureStorage>();

  @override
  Future<Response<AuthenticationResponse>> authenticate(
      LoginModel model) async {
    try {
      var data = {
        "Username": model.username,
        "Password": model.password,
      };
      final http.Response loginResponse = await http.post(
        Uri.parse(config.loginUrl),
        headers: {"content-type": "application/json"},
        body: jsonEncode(data),
      );

      if (loginResponse.statusCode != 200)
        return Response.error(
          "Unexpected Error Has Occured!\n" +
              "STATUS CODE: ${loginResponse.statusCode}\n" +
              "${loginResponse.reasonPhrase}",
        );

      var response =
          AuthenticationResponse.fromJson(jsonDecode(loginResponse.body));

      if (!response.isSuccess) return Response.error(response.errorMessage);

      await _secureStorage.write(key: _tokenKey, value: response.token);

      await _secureStorage.write(
        key: _refreshTokenKey,
        value: response.refreshToken,
      );

      await _secureStorage.write(
        key: _tokenExpireDateKey,
        value: response.tokenExpireDate.millisecondsSinceEpoch.toString(),
      );

      await _secureStorage.write(
        key: _userInfoKey,
        value: jsonEncode(response.userInfo.asJson),
      );
      await _secureStorage.write(key: _usernameKey, value: model.username);
      await _secureStorage.write(key: _passwordKey, value: model.password);

      return Response.success(response);
    } catch (e) {
      return Response.error(
        "Unexpected error has occured while authetication ... \n\n" +
            e.toString(),
      );
    }
  }

  @override
  Future<Response> logOut() async {
    try {
      var token = await _secureStorage.read(key: _tokenKey);
      // var token = _pref.getString(_tokenKey);

      await clearData();

      if (!config.isRootPath(config.logutUrl)) {
        await http.get(
          Uri.parse(config.logutUrl),
          headers: {"Authorization": "Bearer $token"},
        );
      }
      return Response();
    } catch (e) {
      return Response.error(
        "Unexpected error has occured while logout ... \n\n" + e.toString(),
      );
    }
  }

  @override
  Future clearData() async {
    await _secureStorage.deleteAll();
  }

  @override
  Future<Response<AuthenticationResponse>> getAuthData() async {
    if (!await _secureStorage.containsKey(key: _tokenExpireDateKey))
      return Response.error("Token expire date not found!");

    var expireData = await _secureStorage.read(key: _tokenExpireDateKey);
    var tokenExpireDate = DateTime.fromMillisecondsSinceEpoch(
      int.parse(expireData!),
    );

    if (DateTime.now().isAfter(tokenExpireDate))
      return Response.error("Token Expired");

    if (!(await _secureStorage.containsKey(key: _tokenKey)))
      return Response.error("Unauthorized User!");

    var token = await _secureStorage.read(key: _tokenKey);

    if (!(await _secureStorage.containsKey(key: _refreshTokenKey)))
      return Response.error("Refresh token not found! ");

    var refreshToken = await _secureStorage.read(key: _refreshTokenKey);

    if (!await _secureStorage.containsKey(key: _userInfoKey))
      return Response.error("User data not found!");

    var userInfoJson = await _secureStorage.read(key: _userInfoKey);
    var userInfo = DfApiUserInfo.fromJson(
      jsonDecode(userInfoJson!),
    );

    return Response.success(
      AuthenticationResponse(
        token!,
        userInfo,
        refreshToken!,
        tokenExpireDate,
      ),
    );
  }

  @override
  Future<Response<AuthenticationResponse>> refreshToken() async {
    try {
      var authDataResponse = await getAuthData();
      if (!authDataResponse.isSuccess!)
        return Response.error(authDataResponse.errorMessage);

      var data = {"RefreshToken": authDataResponse.value!.refreshToken};

      final http.Response refreshTokenResponse = await http.post(
        Uri.parse(config.refreshTokenUrl),
        headers: {
          "content-type": "application/json",
          "Authorization": "Bearer ${authDataResponse.value!.token}"
        },
        body: jsonEncode(data),
      );

      if (refreshTokenResponse.statusCode != 200)
        return Response.error(
          "Unexpected Error Has Occured!\n" +
              "STATUS CODE: ${refreshTokenResponse.statusCode}\n" +
              "${refreshTokenResponse.reasonPhrase}",
        );

      var response = RefreshTokenResponse.fromJson(
        jsonDecode(refreshTokenResponse.body),
      );

      await _secureStorage.write(key: _tokenKey, value: response.token);
      await _secureStorage.write(
        key: _refreshTokenKey,
        value: response.refreshToken,
      );
      await _secureStorage.write(
        key: _tokenExpireDateKey,
        value: response.tokenExpireDate!.millisecondsSinceEpoch.toString(),
      );

      authDataResponse.value!.token = response.token!;
      authDataResponse.value!.refreshToken = response.refreshToken!;
      authDataResponse.value!.tokenExpireDate = response.tokenExpireDate!;

      return authDataResponse;
    } catch (e) {
      return Response.error(e.toString());
    }
  }

  @override
  Future<Response<LoginModel>> getLoginData() async {
    var isValid = await _secureStorage.containsKey(key: _usernameKey) &&
        await _secureStorage.containsKey(key: _passwordKey);

    if (!isValid) return Response.error("User data not found!");

    var username = await _secureStorage.read(key: _usernameKey);
    var password = await _secureStorage.read(key: _passwordKey);

    return Response.success(LoginModel(username!, password!));
  }
}
